# The software in this package is distributed under the GNU General
# Public License version 2 (with a special exception described below).
#
# A copy of GNU General Public License (GPL) is included in this distribution,
# in the file COPYING.GPL.
#
# As a special exception, if other files instantiate templates or use macros
# or inline functions from this file, or you compile this file and link it
# with other works to produce a work based on this file, this file
# does not by itself cause the resulting work to be covered
# by the GNU General Public License.
#
# However the source code for this file must still be made available
# in accordance with section (3) of the GNU General Public License.
#
# This exception does not invalidate any other reasons why a work based
# on this file might be covered by the GNU General Public License.
#
# Copyright (c) 2016-2022 Intra2net AG <info@intra2net.com>

"""
test_model.py: unit tests for cnfvar/model.py.

Tests classes and functions in cnfvar/model.py

For help see :py:mod:`unittest`
"""

import unittest
import json
from textwrap import dedent
from tempfile import NamedTemporaryFile

from src.cnfvar import CnfList

CNF_TEST_DATA = dedent("""\
    1 USER,1: "jake"
    2    (1) USER_DISABLED,0: "0"
    3    (1) USER_FULLNAME,0: "Jake"
    4    (1) USER_GROUPWARE_FOLDER_DRAFTS,0: "INBOX/Entwürfe"
    5    (1) USER_GROUPWARE_FOLDER_OUTBOX,0: "INBOX/Gesendete Objekte"
    6    (1) USER_GROUPWARE_FOLDER_TRASH,0: "INBOX/Gelöschte Elemente"
    7    (1) USER_GROUP_MEMBER_REF,0: "100"
    8    (1) USER_GROUP_MEMBER_REF,1: "2"
    9    (1) USER_PASSWORD,0: "test1234"
    11 USER,2: "jill"
    12    (11) USER_DISABLED,0: "0"
    13    (11) USER_FULLNAME,0: "Jill"
    14    (11) USER_GROUPWARE_FOLDER_DRAFTS,0: "INBOX/Entwürfe"
    15    (11) USER_GROUPWARE_FOLDER_OUTBOX,0: "INBOX/Gesendete Objekte"
    16    (11) USER_GROUPWARE_FOLDER_TRASH,0: "INBOX/Gelöschte Elemente"
    17    (11) USER_GROUP_MEMBER_REF,0: "100"
    18    (11) USER_GROUP_MEMBER_REF,1: "2"
    19    (11) USER_PASSWORD,0: "test1234"
    74 EMAILFILTER_BAN_FILTERLIST,0: "Vordefiniert: Alles verboten"
    75    (74) EMAILFILTER_BAN_FILTERLIST_ENCRYPTED,0: "BLOCK"
    76    (74) EMAILFILTER_BAN_FILTERLIST_EXTENSIONS,0: ""
    77    (74) EMAILFILTER_BAN_FILTERLIST_FILTER_OFFICE_FILES,0: "BY_FILTERLIST"
    78    (74) EMAILFILTER_BAN_FILTERLIST_MIMETYPES,0: ""
    79       (78) EMAILFILTER_BAN_FILTERLIST_MIMETYPES_NAME,0: "text/plain"
    80       (78) EMAILFILTER_BAN_FILTERLIST_MIMETYPES_NAME,1: "text/html"
    81    (74) EMAILFILTER_BAN_FILTERLIST_MODE,0: "ALLOW"
    82    (74) EMAILFILTER_BAN_FILTERLIST_PREDEFINED_ID,0: "1"
    1196 UPDATE_EXPIRED,0: "0"
    1197 UPDATE_URL_BASE,0: "https://update.intra2net.com/"
    1198 UPDATE_VALIDATION_GROUP,0: "normal"
    1199 UPS_LOCAL_KILLPOWER_ENABLE,0: "1"
    """)


EXPECTED_CNF_DATA = dedent("""\
    1 USER,1: "jake"
    2   (1) USER_DISABLED,0: "0"
    3   (1) USER_FULLNAME,0: "Jake"
    4   (1) USER_GROUPWARE_FOLDER_DRAFTS,0: "INBOX/Entwürfe"
    5   (1) USER_GROUPWARE_FOLDER_OUTBOX,0: "INBOX/Gesendete Objekte"
    6   (1) USER_GROUPWARE_FOLDER_TRASH,0: "INBOX/Gelöschte Elemente"
    7   (1) USER_GROUP_MEMBER_REF,0: "100"
    8   (1) USER_GROUP_MEMBER_REF,1: "2"
    9   (1) USER_PASSWORD,0: "test1234"
    10 USER,2: "jane"
    11   (10) USER_DISABLED,0: "0"
    12   (10) USER_FULLNAME,0: "Jane"
    13   (10) USER_GROUPWARE_FOLDER_DRAFTS,0: "INBOX/Entwürfe"
    14   (10) USER_GROUPWARE_FOLDER_OUTBOX,0: "INBOX/Gesendete Objekte"
    15   (10) USER_GROUPWARE_FOLDER_TRASH,0: "INBOX/Gelöschte Elemente"
    16   (10) USER_GROUP_MEMBER_REF,0: "200"
    17   (10) USER_GROUP_MEMBER_REF,1: "2"
    18   (10) USER_PASSWORD,0: "test1234"
    19   (10) USER_GROUP_MEMBER_REF,2: "5"
    20 EMAILFILTER_BAN_FILTERLIST,0: "Vordefiniert: Alles verboten"
    21   (20) EMAILFILTER_BAN_FILTERLIST_ENCRYPTED,0: "BLOCK"
    22   (20) EMAILFILTER_BAN_FILTERLIST_EXTENSIONS,0: ""
    23   (20) EMAILFILTER_BAN_FILTERLIST_FILTER_OFFICE_FILES,0: "BY_FILTERLIST"
    24   (20) EMAILFILTER_BAN_FILTERLIST_MIMETYPES,0: "" # hey this is a comment
    25     (24) EMAILFILTER_BAN_FILTERLIST_MIMETYPES_NAME,0: "text/plain"
    26     (24) EMAILFILTER_BAN_FILTERLIST_MIMETYPES_NAME,1: "text/html"
    27   (20) EMAILFILTER_BAN_FILTERLIST_MODE,0: "ALLOW"
    28   (20) EMAILFILTER_BAN_FILTERLIST_PREDEFINED_ID,0: "1"
    29 UPDATE_EXPIRED,0: "1"
    30 UPDATE_URL_BASE,0: "https://update.intra2net.com/"
    31 UPDATE_VALIDATION_GROUP,0: "normal"
    32 UPS_LOCAL_KILLPOWER_ENABLE,0: "1"
    """)


class TestModel(unittest.TestCase):
    """Test the multiple capabilities of the CNF modules."""

    def test_querying_and_serialization(self):
        """Test deserializing, querying and serializing a CnfList."""
        cnfs = CnfList.from_cnf_string(CNF_TEST_DATA)

        self._modify(cnfs)

        with NamedTemporaryFile() as tmpfile:
            cnfs.to_cnf_file(tmpfile.name)

            with open(tmpfile.name, "r", encoding="latin1") as f:
                contents = f.read()

        self.assertEqual(contents.splitlines(), EXPECTED_CNF_DATA.splitlines())

        # make sure the result can be parsed again
        CnfList.from_cnf_string(str(contents))

    def test_querying_and_serialization_json(self):
        """Test deserializing, querying and serializing a CnfList as JSON."""
        # first serialize to JSON without renumbering to keep the structure
        cnfs = CnfList.from_cnf_string(CNF_TEST_DATA)

        with NamedTemporaryFile() as tmpfile:
            cnfs.to_json_file(tmpfile.name, renumber=False)

            with open(tmpfile.name, "r") as f:
                contents = f.read()

            # make sure the JSON structure is sane
            json.loads(contents)

        # now asserts that reading from JSON works
        cnfs_2 = CnfList.from_json_string(contents)
        self._modify(cnfs_2)
        cnfs_2.renumber()

        self.assertEqual(str(cnfs_2).splitlines(), EXPECTED_CNF_DATA.splitlines())

        # make sure the result can be parsed again
        CnfList.from_cnf_string(str(cnfs_2))

    def _modify(self, cnfs):
        """
        Make test modifications on the list.

        :param cnfs: list of cnfvars to modify
        :type cnfs: :py:class:`CnfList`

        .. note:: we unify this into a method so we can make sure that the test
        using cnfvar strings and the test using JSON do the exactly same modifications.
        """
        self.assertFalse(cnfs.single_with_name("update_expired").is_enabled())
        cnfs.single_with_name("update_expired") \
            .enable()
        self.assertTrue(cnfs.single_with_name("update_expired").is_enabled())

        user_cnf = cnfs.first_with_value("jill")
        user_cnf.value = "jane"
        user_cnf.children[1].value = "Jane"
        user_cnf.children.first_with_name("user_group_member_ref").value = "200"
        user_cnf.add_children(("USER_GROUP_MEMBER_REF", 5, -1))

        # check correct types and equality
        self.assertEqual(user_cnf.instance, 2)
        self.assertEqual(user_cnf.lineno, 11)

        other_cnf = cnfs.single_with_value("Vordefiniert: Alles verboten")
        other_cnf.children \
                 .where(lambda c: c.name == "emailfilter_ban_filterlist_mimetypes") \
                 .single() \
                 .comment = "hey this is a comment"


if __name__ == '__main__':
    unittest.main()
